/***************************************************************************
 *   Copyright (C) 2004 by Brent Kelly                                     *
 *   hornedgod@hotmail.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kledconfigwindow.h"
#include "kledconfig.h"
#include "kwin.h"
#include "kledconfigwindow.xpm"

KLedConfigWindow::KLedConfigWindow()
 : KMainWindow()
{
    /* set up the little window icon (top left) */
    QPixmap winIcon = QPixmap( kledconfigwindow_xpm );
    KWin::setIcons(this->winId(), winIcon, winIcon);
    
    this->setCaption( QString("KNetworkLED Configuration") );
    
    this->setFixedSize( 230, 180 );
    
    this->m_lblDeviceTextLabel = new QLabel( (QWidget *)this, "devicelabel", 0);
    this->m_lblDeviceTextLabel->move( QPoint(10, 40));
    this->m_lblDeviceTextLabel->setFixedSize( 80, 20 );
    this->m_lblDeviceTextLabel->setText(QString("Device:"));
    
    this->m_lblPollRateTextLabel = new QLabel(  (QWidget *)this, "pollratelabel", 0);
    this->m_lblPollRateTextLabel->move( QPoint(10, 80));
    this->m_lblPollRateTextLabel->setFixedSize( 80, 20 );
    this->m_lblPollRateTextLabel->setText(QString("Poll Rate:"));
    
    this->m_cboDevice = new KComboBox( this, "devicescombo" );
    this->m_cboDevice->setFixedSize( 80, 20 );
    this->m_cboDevice->move( QPoint( 80, 40 ) );
    
    this->m_spnPollRate = new QSpinBox( 100, 1000, 50, this );    
    this->m_spnPollRate->setSuffix( " ms" );
    this->m_spnPollRate->setFixedSize( 80, 20 );
    this->m_spnPollRate->move( QPoint( 80, 80 ) );
    
    this->m_btnSave = new QPushButton( this, "save" );
    this->m_btnSave->setText( QString("Save") );
    this->m_btnSave->move( QPoint( 10, 140 ) );
    this->m_btnSave->show();
    
    this->m_btnCancel = new QPushButton( this, "cancel" );
    this->m_btnCancel->setText( QString( "Cancel" ) );
    this->m_btnCancel->move( QPoint( 120, 140 ) );
    this->m_btnCancel->show();
    
    this->connect(this->m_btnSave, SIGNAL(clicked()), this, SLOT(slot_SaveClicked()) );
    this->connect(this->m_btnCancel, SIGNAL(clicked()), this, SLOT(slot_CancelClicked()) );
    
    /* load configuration data */
    KLedConfig *conf = KLedConfig::Instance();	
    
    /* pollrate from config */
    this->m_spnPollRate->setValue(conf->getPollRate());
    
    /* populate available devices to monitor */
    for ( int i=0; i < conf->getDeviceCount(); i++){
	this->m_cboDevice->insertItem(QString(conf->getDevice(i)), i);
	if ( conf->getChosenDevice() ) {
	    if ( strcmp(conf->getDevice(i), conf->getChosenDevice()) == 0 ) {
		this->m_cboDevice->setCurrentItem(i);
	    }
	}
    }
    
    this->show();
}

/* save configuration data */
void KLedConfigWindow::slot_SaveClicked() {
    KLedConfig *conf = KLedConfig::Instance();
    conf->setChosenDevice((char *)this->m_cboDevice->currentText().ascii());	
    conf->setPollRate(this->m_spnPollRate->value());
    delete this;
}

/* close window ignoring any changes */
void KLedConfigWindow::slot_CancelClicked() {
    delete this;
}

/* dont let this window close entire app
   (which it does for some reason) */
bool KLedConfigWindow::queryClose() {
    delete this;
    //this->destroy();
    return(false);
}

KLedConfigWindow::~KLedConfigWindow()
{
    delete m_lblDeviceTextLabel;
    delete m_lblPollRateTextLabel;
    delete m_spnPollRate;
    delete m_cboDevice; 
    delete m_btnSave;
    delete m_btnCancel;
}
