/***************************************************************************
 *   Copyright (C) 2004 by Brent Kelly                                     *
 *   hornedgod@hotmail.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <ctype.h>

#include "knetworkled.h"
#include "kledwindow.h"

#include <kapplication.h>
#include <klocale.h>
#include <kmessagebox.h>
#include "kledconfig.h"

#include <qtimer.h>
#include <qstring.h>

/* fields in network device file */
#define FIELDCOUNT 12
#define DEVICE_FIELD 1
#define RX_PACKET_FIELD 3
#define TX_PACKET_FIELD 11

KNetworkLed::KNetworkLed()
    : KApplication( true, true )
{
    
    KLedConfig *conf = KLedConfig::Instance();

    // set the shell's ui resource file
    //setXMLFile("knetworkledui.rc");
 
    /* System tray Icon */
    this->m_kledsystemtray = new KLedSystemTray();
    this->setMainWidget( this->m_kledsystemtray );
    connect(this->m_kledsystemtray, SIGNAL(signal_SysTrayDoubleClicked()), this, SLOT(slot_SysTrayDoubleClicked()));
    connect(this->m_kledsystemtray, SIGNAL(quitSelected()), this, SLOT(slot_SysTrayQuit()));
    
    register FILE* devFile;
    if ( ! ( devFile = fopen(conf->getDevFile(), "r"))) {
	this->m_kledsystemtray->setDead("Error reading network device file");
	perror(conf->getDevFile());
    }
    
    /* jump the two header lines in the net device file */
    char buffer[255];
    fgets(buffer, 255, devFile);
    fgets(buffer, 255, devFile);
    char** list;
    fgets(buffer, 255, devFile);
    
    /* store the available devices */
    while ( !feof( devFile ) ) {
 	
	list = split_on_blank(buffer);
	conf->addDevice(list[DEVICE_FIELD]);
	fgets(buffer, 255, devFile);
	
    }
    fclose(devFile);
    
    /* get the selected device */
    if ( !conf->getChosenDevice() ) {
	this->m_kledsystemtray->setDead( "No set device" );
    }
    
     /* get the poll rate */
    if ( !conf->getPollRate() ) {
	conf->setPollRate( 300 );
    }
    
    /* timer for polling network activity */
    this->m_PollTimer = new QTimer(this);
    connect(this->m_PollTimer, SIGNAL(timeout()), this, SLOT(slot_PollForActivity()));
    this->m_PollTimer->start(conf->getPollRate(), false);
    /*because there's no method of pulling the timer's current
      pollrate, save the current, so we know to change it if the 
      user alters the poll rate while running */
    this->m_CurrentPollRate = conf->getPollRate();
    
    /* status window */
    this->m_kledwindow = new KLedwindow;

}

/* bring up config window if systray dbl clicked */
void KNetworkLed::slot_SysTrayDoubleClicked() {
    if ( this->m_kledwindow->isHidden() ) {
	this->m_kledwindow->updateStatus( this->m_kledsystemtray->m_PrevTX , this->m_kledsystemtray->m_PrevRX);
	this->m_kledwindow->show();
    } else {
	this->m_kledwindow->hide();
    }	
}

void KNetworkLed::slot_SysTrayQuit() {
    
    /* quits application if systemtray is killed */
    this->quit();
    
}

int KNetworkLed::slot_PollForActivity() {
    register FILE* devFile;
    char buffer[255];
    int iDone = 0;
    KLedConfig *conf = KLedConfig::Instance();

    /* Test to see if poll interval has been changed */
    if ( this->m_CurrentPollRate != conf->getPollRate() ) {
    	this->m_PollTimer->changeInterval(conf->getPollRate());
	this->m_CurrentPollRate = conf->getPollRate();
    }
    
    /* Make sure devices are configured */
    if ( !conf->getChosenDevice() || !conf->getPollRate() ) {
	printf("'%s' and '%d'\n", conf->getChosenDevice(), conf->getPollRate() );
	this->m_kledsystemtray->setDead("Please configure KNetworkLED");
	return(0);
    }
    
    /* open network dev file */
    if (! (devFile = fopen(conf->getDevFile(), "r")) ) {
	this->m_kledsystemtray->setDead("Error reading network device file");
	perror(conf->getDevFile());
	return(0);
    }

    
    /* jump the two header lines in the net device file */
    fgets(buffer, 255, devFile);
    fgets(buffer, 255, devFile);
    
    /* loop thru the rest of the lines */
    while ( iDone == 0 ) {

	/* will hold line of text from dev file to test */
	QString szLine;
	
	/* grab another line from the
	   net/dev file and slap it in a QString */
	fgets(buffer, 255, devFile);
	szLine = (const char *)buffer;
	
	/* see if we hit the end of the net/dev file */
	if ( !feof( devFile ) ) {

	    /* check to see if its the interface the user chose to monitor */
	    if ( szLine.find(conf->getChosenDevice(), 0) != -1 ) {
		
		/* FROM tleds (changed a bit)*/ 
		char** list;
		register unsigned long received = 0;
		register unsigned long transmitted = 0;
		list = split_on_blank((char *)szLine.ascii());
		
		//packets:
		received = atol(list[RX_PACKET_FIELD]);
		transmitted = atol(list[TX_PACKET_FIELD]);
		
		/* let systemtray know what's going on so it can decide to update or not */
		this->m_kledsystemtray->setStatus( received, transmitted );
		/* if the ledwindow is open, update the status */
		if ( ! this->m_kledwindow->isHidden() ) {
		    this->m_kledwindow->updateStatus(transmitted, received);
		}
		
	    }
	    
	} else {
	    
	    /* EOF */
	    iDone = 1;
	    break;	//dont think this is necessary ( with the iDone =1 and all)
	}
    
    }
    
    /* polling complete */
    fclose(devFile);
    return(1);
}

/* FROM tleds */
char** KNetworkLed::split_on_blank (char* line) {
	/*
	Has a "bug". If tried to monitor network traffic e.g. on "dummy".
	Look at the end of this file for example /proc/net/dev listing.
	*/
	
	static char*    list [FIELDCOUNT] = {};
	register int    i;
	
	i = 0;
	goto middle;    /* speed(?) hack */
	for (; i < FIELDCOUNT; i++) {
			while (isblank(*line))
				line++;			
middle:
			list[i] = line;
			while (! isblank(*line) && *line != ':' && *line != '\n')
				line++;
			*(line++) = '\0';
	}
	return list;
}


KNetworkLed::~KNetworkLed()
{
    delete(this->m_kledwindow);
    delete(this->m_kledsystemtray);
}

#include "knetworkled.moc"
